/*
 * Copyright 2006 Alexander Kornrumpf
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


package org.jedit.syntax.flex;

import org.jedit.syntax.flex.LexicalScanner;
import org.jedit.syntax.tokenmarker.Token;

import javax.swing.text.Segment;

import java.io.CharArrayReader;

%%

%class BatchFileScanner
%extends LexicalScanner
%public

%{	
	private int lastOffset;
	
	public int yyeof() {return YYEOF;};
%}

%function nextToken
%int

%unicode
%ignorecase

EOL = \r|\n|\r\n
NoSpace=[^\r\n \t\f]
Whitespace= {EOL} | [ \t\f]

%state KEYWORD2
%state LITERAL1

%%
<YYINITIAL> CALL|CHOICE|CLS|COPY|DO|ECHO|ERRORLEVEL|EXISTS|EXIT|FOR|IF|IN|NOT|PAUSE|SET|SHIFT|START
								{return Token.KEYWORD1;}
<YYINITIAL>	REM~{EOL}			{return Token.COMMENT1;}
<YYINITIAL> %%{NoSpace}*    	{return Token.KEYWORD2;}
<YYINITIAL> :{NoSpace}*			{return Token.LABEL;}
<YYINITIAL> %					{
									yybegin(KEYWORD2);
									lastOffset=zzStartRead;
								}
<YYINITIAL> \"~\"			    {
									return Token.LITERAL1;
								}
<YYINITIAL> \"[^\"]*{EOL}		{
									yybegin(LITERAL1);
									return Token.LITERAL1;
								}

<YYINITIAL> [^\r\n \t\f%\"]*  	{return Token.NULL;}
<YYINITIAL> {Whitespace}*		{return Token.NULL;}

<KEYWORD2> 
{
	%							{
									zzStartRead=lastOffset;
									yybegin(YYINITIAL);
									return Token.KEYWORD2;
								}
	{Whitespace}				{
									zzStartRead=lastOffset;
									yybegin(YYINITIAL);
									return Token.INVALID;
								}
	[^%]						{/*ignore*/}
}
<LITERAL1> 
{
	~\"							{
									yybegin(YYINITIAL);
									return Token.LITERAL1;
								}
	[^\"]*{EOL}					{

									return Token.LITERAL1;
								}
}
